/*!---------------------------------------------------------
* Copyright (C) Microsoft Corporation. All rights reserved.
*--------------------------------------------------------*/
/// <reference path="../declare/node.d.ts" />
/// <reference path="../declare/atom-browser.d.ts" />
'use strict';
var env = require('../env');
var BrowserWindow = require('browser-window');
var ipc = require('ipc');
function configure(bus) {
    var cache = Object.create(null);
    ipc.on('git:askpass', function (event, result) {
        cache[result.id].credentials = result.credentials;
    });
    bus.addListener('git:askpass', function (context) {
        var cachedResult = cache[context.id];
        if (typeof cachedResult !== 'undefined') {
            bus.emit('git:askpass:' + context.id, cachedResult.credentials);
            return;
        }
        // TODO@Joao
        if (context.command === 'fetch') {
            bus.emit('git:askpass:' + context.id, { id: context.id, credentials: { username: '', password: '' } });
            return;
        }
        var win = new BrowserWindow({
            'always-on-top': true,
            'skip-taskbar': true,
            resizable: false,
            width: 450,
            height: env.isWindows ? 280 : 260,
            show: true
        });
        win.setMenuBarVisibility(false);
        cache[context.id] = {
            window: win,
            credentials: null
        };
        win.loadUrl('file://' + __dirname.replace(/\\/g, '/') + '/index.html');
        win.webContents.executeJavaScript('init(' + JSON.stringify(context) + ')');
        win.once('closed', function () {
            bus.emit('git:askpass:' + context.id, cache[context.id].credentials);
            setTimeout(function () {
                delete cache[context.id];
            }, 1000 * 10);
        });
    });
}
exports.configure = configure;
