/*---------------------------------------------------------
 * Copyright (C) Microsoft Corporation. All rights reserved.
 *--------------------------------------------------------*/
/// <reference path="./declare/node.d.ts" />
'use strict';
var __extends = this.__extends || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
var net = require('net');
var events = require('./events');
function connectBus(bus, socket) {
    var isConnected = true;
    var eventStack = [];
    socket.setEncoding('utf8');
    socket.setNoDelay();
    function emit(type, data) {
        if (data === void 0) { data = null; }
        if (isConnected) {
            eventStack.push({
                type: type,
                data: data
            });
        }
        bus.emit(type, data);
    }
    var unbind = bus.addBulkListener(function (events) {
        events.forEach(function (event) {
            var type = event.getType();
            var data = event.getData();
            var lastKnownEvent = eventStack[eventStack.length - 1];
            if (lastKnownEvent && lastKnownEvent.type === type && lastKnownEvent.data === data) {
                eventStack.pop();
            }
            else {
                socket.write(JSON.stringify({
                    type: type,
                    data: data
                }) + '\n');
            }
        });
    });
    socket.on('data', function (data) {
        data.split('\n').filter(function (data) { return !!data; }).map(function (data) { return JSON.parse(data); }).forEach(function (event) { return emit(event.type, event.data); });
    });
    socket.on('error', function (err) {
        emit('error', err);
    });
    socket.on('close', function () {
        socket.removeAllListeners(); // TODO@Alex method does not exist?
        isConnected = false;
        unbind();
        emit('close');
    });
    return function () {
        socket.close(); // TODO@Alex method does not exist?
    };
}
var ConnectedBus = (function (_super) {
    __extends(ConnectedBus, _super);
    function ConnectedBus(socket) {
        _super.call(this);
        this.socket = socket;
        connectBus(this, socket);
    }
    ConnectedBus.prototype.disconnect = function () {
        this.socket.end();
    };
    return ConnectedBus;
})(events.EventEmitter);
exports.ConnectedBus = ConnectedBus;
function connect(hook, callback) {
    var socket = net.createConnection(hook, function () { return callback(null, new ConnectedBus(socket)); });
    return socket;
}
exports.connect = connect;
function publish(hook, bus) {
    var server = net.createServer(function (s) { return connectBus(bus, s); });
    server.listen(hook);
    return server;
}
exports.publish = publish;
