/*---------------------------------------------------------
 * Copyright (C) Microsoft Corporation. All rights reserved.
 *--------------------------------------------------------*/
/// <reference path="../../declares.d.ts" />
'use strict';
define(["require", "exports", '../rubyDef', 'monaco-testing'], function (require, exports, rubyDef, T) {
    var tokenize = T.createTokenize(rubyDef.language);
    var assertTokens = T.assertTokens;
    T.module('Ruby Colorizer');
    T.test('Keywords', function () {
        var tokens = tokenize('class Klass def init() end').tokens;
        assertTokens(tokens, [
            { startIndex: 0, type: 'keyword.class.ruby' },
            { startIndex: 5, type: '' },
            { startIndex: 6, type: 'constructor.identifier.ruby' },
            { startIndex: 11, type: '' },
            { startIndex: 12, type: 'keyword.def.ruby' },
            { startIndex: 15, type: '' },
            { startIndex: 16, type: 'identifier.ruby' },
            { startIndex: 20, type: 'delimiter.parenthesis.ruby' },
            { startIndex: 21, type: 'delimiter.parenthesis.ruby' },
            { startIndex: 22, type: '' },
            { startIndex: 23, type: 'keyword.def.ruby' }
        ]);
    });
    T.test('Single digit', function () {
        var tokens = tokenize('x == 1 ').tokens;
        assertTokens(tokens, [
            { startIndex: 0, type: 'identifier.ruby' },
            { startIndex: 1, type: '' },
            { startIndex: 2, type: 'operator.ruby' },
            { startIndex: 4, type: '' },
            { startIndex: 5, type: 'number.ruby' },
            { startIndex: 6, type: '' }
        ]);
    });
    T.test('Regex', function () {
        var tokens = tokenize('text =~ /Ruby/').tokens;
        assertTokens(tokens, [
            { startIndex: 0, type: 'identifier.ruby' },
            { startIndex: 4, type: '' },
            { startIndex: 5, type: 'operator.ruby' },
            { startIndex: 7, type: '' },
            { startIndex: 8, type: 'regexp.delim.ruby' },
            { startIndex: 9, type: 'regexp.ruby' },
            { startIndex: 13, type: 'regexp.delim.ruby' }
        ]);
        tokens = tokenize('text.sub!(/Rbuy/, "Ruby")').tokens;
        assertTokens(tokens, [
            { startIndex: 0, type: 'identifier.ruby' },
            { startIndex: 4, type: '' },
            { startIndex: 5, type: 'identifier.ruby' },
            { startIndex: 9, type: 'delimiter.parenthesis.ruby', 'bracket': 1 },
            { startIndex: 10, type: 'regexp.delim.ruby', 'bracket': 1 },
            { startIndex: 11, type: 'regexp.ruby' },
            { startIndex: 15, type: 'regexp.delim.ruby', 'bracket': -1 },
            { startIndex: 16, type: 'delimiter.ruby' },
            { startIndex: 17, type: '' },
            { startIndex: 18, type: 'string.d.delim.ruby', 'bracket': 1 },
            { startIndex: 19, type: 'string.$S2.ruby' },
            { startIndex: 23, type: 'string.d.delim.ruby', 'bracket': -1 },
            { startIndex: 24, type: 'delimiter.parenthesis.ruby', 'bracket': -1 }
        ]);
        // make sure that division does not match regex
        tokens = tokenize('a / b').tokens;
        assertTokens(tokens, [
            { startIndex: 0, type: 'identifier.ruby' },
            { startIndex: 1, type: '' },
            { startIndex: 2, type: 'operator.ruby' },
            { startIndex: 3, type: '' },
            { startIndex: 4, type: 'identifier.ruby' },
        ]);
    });
    T.test('Heredoc', function () {
        var lineTokens = tokenize('<<HERE');
        assertTokens(lineTokens.tokens, [{ startIndex: 0, type: 'string.heredoc.delimiter.ruby', 'bracket': 1 }]);
        lineTokens = tokenize('do some string', lineTokens.endState);
        assertTokens(lineTokens.tokens, [{ startIndex: 0, type: 'string.heredoc.ruby' }]);
        lineTokens = tokenize('HERE', lineTokens.endState);
        assertTokens(lineTokens.tokens, [{ startIndex: 0, type: 'string.heredoc.delimiter.ruby', 'bracket': -1 }]);
        var lineTokens = tokenize('x <<HERE');
        assertTokens(lineTokens.tokens, [
            { startIndex: 0, type: 'identifier.ruby' },
            { startIndex: 1, type: 'string.heredoc.delimiter.ruby', 'bracket': 1 },
        ]);
        lineTokens = tokenize('do some string', lineTokens.endState);
        assertTokens(lineTokens.tokens, [{ startIndex: 0, type: 'string.heredoc.ruby' }]);
        lineTokens = tokenize('HERE', lineTokens.endState);
        assertTokens(lineTokens.tokens, [{ startIndex: 0, type: 'string.heredoc.delimiter.ruby', 'bracket': -1 }]);
        var lineTokens = tokenize('x<<HERE');
        assertTokens(lineTokens.tokens, [
            { startIndex: 0, type: 'identifier.ruby' },
            { startIndex: 1, type: 'operator.ruby' },
            { startIndex: 3, type: 'constructor.identifier.ruby' }
        ]);
        var lineTokens = tokenize('x<<-HERE');
        assertTokens(lineTokens.tokens, [
            { startIndex: 0, type: 'identifier.ruby' },
            { startIndex: 1, type: 'string.heredoc.delimiter.ruby' },
        ]);
    });
});
