/*---------------------------------------------------------
 * Copyright (C) Microsoft Corporation. All rights reserved.
 *--------------------------------------------------------*/
/// <reference path="../../declare/node.d.ts" />
/// <reference path="../../declare/http-proxy-agent.d.ts" />
/// <reference path="../../declare/https-proxy-agent.d.ts" />
'use strict';
define(["require", "exports", '../../lib/system', 'https', 'http', 'url', 'http-proxy-agent', 'https-proxy-agent'], function (require, exports, winjs, https, http, url, HttpProxyAgent, HttpsProxyAgent) {
    var proxyConfiguration = null;
    function configure(proxyURI) {
        proxyConfiguration = proxyURI;
    }
    exports.configure = configure;
    function getProxyURI(uri) {
        var proxyURI = proxyConfiguration;
        if (!proxyURI) {
            if (uri.protocol === 'http:') {
                proxyURI = process.env.HTTP_PROXY || process.env.http_proxy || null;
            }
            else if (uri.protocol === 'https:') {
                proxyURI = process.env.HTTPS_PROXY || process.env.https_proxy || process.env.HTTP_PROXY || process.env.http_proxy || null;
            }
        }
        return proxyURI;
    }
    function getProxyAgent(uri) {
        var proxyURI = getProxyURI(uri);
        if (proxyURI) {
            var proxyEndpoint = url.parse(proxyURI);
            switch (proxyEndpoint.protocol) {
                case 'http:':
                case 'https:':
                    return uri.protocol === 'http:' ? new HttpProxyAgent(proxyURI) : new HttpsProxyAgent(proxyURI);
            }
        }
        return void 0;
    }
    function xhr(options) {
        var request;
        return new winjs.Promise(function (c, e, p) {
            var endpoint = url.parse(options.url);
            var opts = {
                hostname: endpoint.hostname,
                port: endpoint.port ? parseInt(endpoint.port) : endpoint.protocol === 'https:' ? 443 : 80,
                path: endpoint.path,
                method: options.type || 'GET',
                headers: options.headers,
                agent: getProxyAgent(endpoint)
            };
            if (options.user && options.password) {
                opts.auth = options.user + ':' + options.password;
            }
            var httpLibrary = endpoint.protocol === 'https:' ? https : http;
            request = httpLibrary.request(opts, function (res) {
                var data = [];
                res.on('data', function (chunk) {
                    data.push(chunk);
                });
                res.on('end', function () {
                    if (options.followRedirects > 0 && (res.statusCode >= 300 && res.statusCode <= 303 || res.statusCode === 307)) {
                        var location = res.headers['location'];
                        if (location) {
                            var newOptions = {
                                type: options.type,
                                url: location,
                                user: options.user,
                                password: options.password,
                                responseType: options.responseType,
                                headers: options.headers,
                                timeout: options.timeout,
                                followRedirects: options.followRedirects - 1,
                                data: options.data
                            };
                            xhr(newOptions).then(c, e, p);
                            return;
                        }
                    }
                    var response = {
                        responseText: data.join(''),
                        status: res.statusCode
                    };
                    if ((res.statusCode >= 200 && res.statusCode < 300) || res.statusCode === 1223) {
                        c(response);
                    }
                    else {
                        e(response);
                    }
                });
            });
            request.on('error', function (err) {
                var message;
                if (opts.agent) {
                    message = 'Unable to to connect to ' + options.url + ' through proxy ' + getProxyURI(endpoint) + '. Error: ' + err.message;
                }
                else {
                    message = 'Unable to to connect to ' + options.url + '. Error: ' + err.message;
                }
                var response = {
                    responseText: message,
                    status: 404
                };
                e(response);
            });
            if (options.timeout) {
                request.setTimeout(options.timeout);
            }
            if (options.data) {
                request.write(options.data);
            }
            request.end();
        }, function () {
            if (request) {
                request.abort();
            }
        });
    }
    exports.xhr = xhr;
});
