/*---------------------------------------------------------
 * Copyright (C) Microsoft Corporation. All rights reserved.
 *--------------------------------------------------------*/
'use strict';
define(["require", "exports", '../../lib/system', './fileSearch', './textSearch'], function (require, exports, winjs, fileSearch, textSearch) {
    var RawSearchService = (function () {
        function RawSearchService() {
        }
        RawSearchService.prototype.fileSearch = function (config) {
            config.filePatterns = config.filePatterns && config.filePatterns.length > 0 ? config.filePatterns : [{ pattern: '' /* All files */ }];
            var engine = new fileSearch.Engine(config);
            return this.doSearch(engine);
        };
        RawSearchService.prototype.textSearch = function (config) {
            config.filePatterns = config.filePatterns && config.filePatterns.length > 0 ? config.filePatterns : [{ pattern: '' /* All files */ }];
            var engine = new textSearch.Engine(config, new fileSearch.FileWalker({
                rootResources: config.rootResources,
                includePattern: config.includePattern,
                excludePattern: config.excludePattern,
                filePatterns: config.filePatterns
            }));
            return this.doSearch(engine);
        };
        RawSearchService.prototype.doSearch = function (engine) {
            return new winjs.PPromise(function (c, e, p) {
                engine.search(function (match) {
                    if (match) {
                        p(match);
                    }
                }, function (progress) {
                    p(progress);
                }, function (error, isLimitHit) {
                    if (error) {
                        e(error);
                    }
                    else {
                        c({
                            limitHit: isLimitHit
                        });
                    }
                });
            }, function () { return engine.cancel(); });
        };
        return RawSearchService;
    })();
    exports.RawSearchService = RawSearchService;
});
